import tkinter
import math
import cmath
import tkinter.filedialog
import tkinter.messagebox

from Graphenprojekt.controller import *

## Benjamin Molls Entwurf, aber ohne die Klasse Farben
## also sind englische Farbnamen oder Werte wie '#f0a' zwingend

class _Leinwand:
    """
    Eine Leinwand, auf die gezeichnet werden kann. Die einzelnen Formen
    benutzen diese Leinwand, um sich darauf selbst zu zeichnen. 
    """
    _instance=None

    def __init__(self,width=800,height=800,bg='white'):
        """
        Erstellt das Fenster mit der Leinwand für das Möbelprojekt.
        width: die Breite der Leinwand; Standard: '18c' für 18cm
        height: die Höhe der Leinwand; Standard: '14c' für 14cm
        bg: die Hintergrundfarbe; Standard: 'weiss'
        """
        if _Leinwand._instance:
            raise Error("Nur eine Leinwand erlaubt!")
        else:
            _Leinwand._instance=self
            self.fenster=tkinter.Tk()
            self.fenster.title('Graphendarstellung')
            self.canvas=tkinter.Canvas(self.fenster,width=width,height=height,bg=bg)
            self.canvas.pack()

    ### Threadbereich
    def Action(self, sec, anzahl):
        """iterative Anwendung:"""
        for name in ['A','B','C','D','E','F']:
            self.aendereFuellFarbe(Knoten, name, 'red')
            time.sleep(sec)

    def starte(self):
        _thread.start_new_thread(self.Action, (2, 8))

    def zeichne(self,form):
        """
        Zeichnet die Form auf die Leinwand.
        form: die zu zeichnenende Form
        """
        return form.zeichne(self)

    def loesche(self,item):
        return self.canvas.delete(item)

    def polygon(self,*coords,**kw):
        '''gibt die ID zurueck'''
        return self.canvas.create_polygon(*coords,**kw)

    def ellipse(self,x1,y1,x2,y2,**kw):
        '''gibt die ID zurueck'''
        return self.canvas.create_oval(x1,y1,x2,y2,**kw)

    def linie(self,x1,y1,x2,y2,**kw):
        '''gibt die ID zurueck'''
        return self.canvas.create_line(x1,y1,x2,y2,**kw)

    def text(self,x1,y1,derText,**kw):
        '''gibt die ID zurueck'''
        return self.canvas.create_text(x1,y1,text=derText,**kw)

    def skalieren(self,item,x,y,xFaktor,yFaktor):
        return self.canvas.scale(item,x,y,xFaktor,yFaktor)

    def bewegen(self,item,x,y):
        return self.canvas.move(item,x,y)
    
    def farbeAendern(self,item,neueFarbe):
         return self.canvas.itemconfig(item,fill=neueFarbe)

    def randfarbeAendern(self,item,neueFarbe):
       return self.canvas.itemconfig(item,outline=neueFarbe)

    def drehe(self,item,x,y,winkel):
        
        cwinkel=cmath.exp(winkel*math.pi/180*1j)
        coords=self.canvas.coords(item)
        newcoords=[]
        zentrum=complex(x,y)
        X=None
        for Y in coords:
            if X is None:
                X=Y
            else:
                neuer_punkt=cwinkel*(complex(X,Y)-zentrum)+zentrum
                newcoords.append(neuer_punkt.real)
                newcoords.append(neuer_punkt.imag)
                X=None
        self.canvas.coords(item,*newcoords)

    def bildExport(self):
        if _Leinwand._instance==None: return
        dateiname=tkinter.filedialog.asksaveasfilename(\
            parent=self.fenster,
            initialfile="Bild_.ps",
            defaultextension='ps',
            initialdir='Bilder',
            title="Bild speichern")
        if dateiname in ['',()]:
            tkinter.messagebox.askokcancel('Bild nicht gespeichert','Achtung !',icon='warning')
        else:
            self.canvas.postscript(file=dateiname)

def Leinwand(singleton=_Leinwand()):
    """
    Liefert die Leinwand. Hiervon darf es nur ein einziges Objekt geben,
    deswegen liefert diese Funktion bei jedem Aufruf dieselbe Leinwand
    zurück. Dies ist nötig, damit die einzelnen Objekte auf dieselbe
    Leinwand gezeichnet werden anstatt eine Leinwand pro Objekt zu haben.
    """
    return singleton
